package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.params.CourseEvaluationParam;
import com.qmth.distributed.print.business.bean.result.CourseEvaluationResult;
import com.qmth.distributed.print.business.entity.CourseEvaluation;
import com.qmth.distributed.print.business.entity.TeachCourse;
import com.qmth.distributed.print.business.enums.CourseEvaluationTypeEnum;
import com.qmth.distributed.print.business.enums.CourseSettingTypeEnum;
import com.qmth.distributed.print.business.mapper.CourseEvaluationMapper;
import com.qmth.distributed.print.business.service.CourseEvaluationService;
import com.qmth.distributed.print.business.service.TeachCourseService;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 课程评价方式 服务实现类
 * </p>
 *
 * @author CaoZixuan
 * @since 2024-02-22
 */
@Service
public class CourseEvaluationServiceImpl extends ServiceImpl<CourseEvaluationMapper, CourseEvaluation> implements CourseEvaluationService {

    @Resource
    private TeachCourseService teachCourseService;

    @Override
    public List<CourseEvaluationResult> findCourseEvaluationList(Long examId, String courseCode, Long userId) {
        TeachCourse teachCourse = teachCourseService.findByExamIdCourseCodeAndUserId(examId, courseCode, userId);
        return this.baseMapper.findCourseEvaluationList(teachCourse.getId());
    }

    @Transactional
    @Override
    public void saveCourseEvaluation(CourseEvaluationParam courseEvaluationParam, Long userId) {
        Long examId = courseEvaluationParam.getExamId();
        String courseCode = courseEvaluationParam.getCourseCode();
        String evaluation = courseEvaluationParam.getEvaluation();
        String evaluationDesc = courseEvaluationParam.getEvaluationDesc();
        CourseEvaluationTypeEnum type = courseEvaluationParam.getType();

        TeachCourse teachCourse = teachCourseService.findByExamIdCourseCodeAndUserId(examId, courseCode, userId);
        Long teachCourseId = teachCourse.getId();
        if (this.count(
                new QueryWrapper<CourseEvaluation>().lambda().eq(CourseEvaluation::getTeachCourseId, teachCourseId)
                        .eq(CourseEvaluation::getEvaluation, evaluation)) > 0) {
            throw ExceptionResultEnum.ERROR.exception("评价方式重复");
        }

        CourseEvaluation courseEvaluation = new CourseEvaluation();
        courseEvaluation.setTeachCourseId(teachCourseId);
        courseEvaluation.setSchoolId(teachCourse.getSchoolId());
        courseEvaluation.setExamId(examId);
        courseEvaluation.setCourseCode(courseCode);
        courseEvaluation.setUserId(userId);
        courseEvaluation.setType(type);
        courseEvaluation.setEvaluation(evaluation);
        courseEvaluation.setEvaluationDesc(evaluationDesc);
        courseEvaluation.insertInfo(userId);
        this.save(courseEvaluation);
    }

    @Transactional
    @Override
    public void deleteCourseEvaluation(Long id) {
        CourseEvaluation courseEvaluation = this.getById(id);
        if (Objects.nonNull(courseEvaluation)) {
            if (CourseEvaluationTypeEnum.DEFAULT.equals(courseEvaluation.getType())) {
                throw ExceptionResultEnum.ERROR.exception(String.format("[%s]的评价方式不能删除", CourseEvaluationTypeEnum.DEFAULT.getType()));
            }
            Long teachCourseId = courseEvaluation.getTeachCourseId();
            teachCourseService.clearCourseSetting(teachCourseId, CourseSettingTypeEnum.COURSE_EVALUATION);
            this.removeById(id);
        }
    }

    @Transactional
    @Override
    public void createDefaultCourseEvaluation(Long examId, String courseCode, Long userId) {
        // 创建默认评价方式
        CourseEvaluationTypeEnum type = CourseEvaluationTypeEnum.DEFAULT;
        CourseEvaluationParam courseEvaluationParam = new CourseEvaluationParam();
        courseEvaluationParam.setExamId(examId);
        courseEvaluationParam.setCourseCode(courseCode);
        courseEvaluationParam.setEvaluation(type.getName());
        courseEvaluationParam.setEvaluationDesc(type.getValue());
        courseEvaluationParam.setType(type);
        this.saveCourseEvaluation(courseEvaluationParam, userId);
    }
}
